from docx.shared  import Inches


DOC_FONT_SETTINGS = {
    "BOOL_SETTINGS": [
        # 加粗
        "bold", 
        # 斜体
        "italic", 
        # 下划线
        "underline", 
        # 以下的没看出效果，先列出来
        "double_strike", 
        "emboss", 
        "hidden", 
        "imprint", 
        "outline", 
        "rtl", 
        "shadow", 
        "snap_to_grid", 
        "spec_vanish", 
        "strike", 
        "subscript", 
        "superscript", 
        "web_hidden"
    ]

}